<?php
namespace Webkul\SalesforceConnector\Controller\Adminhtml\Products;

class GetAllExportProducts extends \Magento\Backend\App\Action
{
    protected $_resultPageFactory = false;
    protected $_connection = false;
    protected $_helper;
    protected $_json;
    protected $_categoryModel;
    protected $_productModel;
    protected $_catalogCategoryModel;
    protected $_catalogProductModel;
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Webkul\SalesforceConnector\Helper\Connection $connection,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Webkul\SalesforceConnector\Helper\Data $helper,
        \Webkul\SalesforceConnector\Model\ProductFactory $product,
        \Magento\Catalog\Model\ProductFactory $productCategory,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory
    ) {
    
        parent::__construct($context);
        $this->_resultPageFactory = $resultPageFactory;
        $this->_connection = $connection;
        $this->_helper = $helper;
        $this->_json = $jsonHelper;
        $this->_productModel = $product;
        $this->_catalogProductModel = $productCategory;
    }

    public function execute()
    {
        $mapProducts = $exportIds = [];
        if (\Webkul\SalesforceConnector\Helper\Connection::checkConnectionStatus()) {
            $productModel = $this->_productModel->create();
            $catalogProductModel = $this->_catalogProductModel->create();
            
            $productCollection = $productModel
                                    ->getCollection()
                                    ->addFieldToSelect('magento_id')
                                    ->addFieldToFilter('sync_status', '1')
                                    ->getData();
            $mapProducts = array_column($productCollection, 'magento_id');
            
            if ($mapProducts) {
                $collection = $catalogProductModel->getCollection()
                ->addAttributeToFilter('entity_id', ['nin' => $mapProducts])
                ->addAttributeToSelect('entity_id');
            } else {
                $collection = $catalogProductModel->getCollection()
                ->addAttributeToSelect('entity_id');
            }
            $exportIds = $collection->getAllIds();
            
            if (count($exportIds) == 0) {
                    $this->messageManager->addSuccess(__("All Products are Already exported at Salesforce."));
            }
        } else {
            $this->messageManager->addError(__("Check Connection"));
        }
        $this->getResponse()->setHeader('Content-type', 'application/json');
        $this->getResponse()->setBody($this->_json->jsonEncode($exportIds));
    }

    /*
     * Check permission via ACL resource
     */
    // protected function _isAllowed()
    // {
    //     return $this->_authorization->isAllowed('Webkul_Eshopsync::products_getallexportproducts');
    // }
}
