<?php

/**
 * * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Webkul\SalesforceConnector\Controller\Adminhtml\System\Config\Connection;

use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\App\PageCache\Version;
use Magento\Framework\App\Cache\TypeListInterface;
use Magento\Framework\App\Cache\Frontend\Pool;

class OrderFieldMap extends \Webkul\SalesforceConnector\Controller\Adminhtml\System\Config\Connection
{
    /**
     * @var JsonFactory
     */
    protected $_json;
    protected $writerInterface;
    protected $_curl;
    protected $cacheTypeList;
    protected $cacheFrontendPool;
    protected $_helper;
    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Webkul\Eshopsync\Helper\Connection $connection
     * @param \Webkul\Eshopsync\Helper\Data $helper
     * @param \Magento\Config\Model\ResourceModel\Config $resourceConfig
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Magento\Framework\HTTP\Client\Curl $curl
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Magento\Framework\App\Config\Storage\WriterInterface $writerInterface,
        TypeListInterface $cacheTypeList,
        Pool $cacheFrontendPool,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Webkul\SalesforceConnector\Helper\Connection $helper
    ) {

        parent::__construct($context);
        $this->writerInterface = $writerInterface;
        $this->_json = $jsonHelper;
        $this->_curl = $curl;
        $this->_helper = $helper;
        $this->cacheTypeList = $cacheTypeList;
        $this->cacheFrontendPool = $cacheFrontendPool;
    }

    public function execute()
    { 
        
        $post = $this->getRequest()->getPostValue();
        try{
            $this->writerInterface->save('sfConnect/ord/orderFieldMap', $this->_json->jsonEncode(isset($post['orderMappingData'])?$post['orderMappingData']:null));             
            $this->cacheTypeList->cleanType('config');
            $this->messageManager->addSuccess(__("Mapping Saved"));
            foreach ($this->cacheFrontendPool as $cacheFrontend) {
                $cacheFrontend->getBackend()->clean();
            }           
            $this->getResponse()->setHeader('Content-type', 'application/json');
            $this->getResponse()->setBody(1);
        }
        catch (\Exception $e){
            $this->messageManager->addError(__("Mapping Failed"));
            $this->getResponse()->setHeader('Content-type', 'application/json');
            $this->getResponse()->setBody(0);
        }
        
        
    }

}
