<?php

/**
 * Webkul Software.
 *
 * @category  Webkul
 *
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Helper;

use Magento\Framework\App\Helper\AbstractHelper;

// use \Webkul\SalesforceConnector\Logger\EshopsyncLogger as Logger;

/**
 * Eshopsync category helper.
 */
class Category extends AbstractHelper
{
    /**
     * @var \Magento\Framework\Event\Manager
     */
    protected $_eventManager;

    /**
     * @var \Magento\Catalog\Helper\Image
     */
    protected $_image;

    /**
     * @var \Magento\Catalog\Model\CategoryFactory
     */
    protected $_catalogCategory;

    /**
     * @var \Webkul\Eshopsync\Model\CategoryFactory
     */
    protected $_eshopsyncCategory;

    /**
     * @var \Webkul\Eshopsync\Helper\Data
     */
    protected $_helperData;

    /**
     * @var \Webkul\Eshopsync\Helper\Connection
     */
    protected $_helperConnection;

    /**
     * @var Logger
     */
    // protected $_logger;
    protected $_registry;

    /**
     * @param Magento\Framework\App\Helper\Context   $context
     * @param Magento\Framework\Event\Manager        $eventManager
     * @param Magento\Catalog\Model\CategoryFactory  $catalogCategory
     * @param Magento\Catalog\Helper\Image           $image
     * @param Webkul\Eshopsync\Model\CategoryFactory $eshopsyncCategory
     * @param Webkul\Eshopsync\Helper\Data           $helperData
     * @param Webkul\Eshopsync\Helper\Connection     $helperConnection
     * @param Logger                                 $logger
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Event\Manager $eventManager,
        \Magento\Catalog\Model\CategoryFactory $catalogCategory,
        \Magento\Catalog\Helper\Image $image,
        \Webkul\SalesforceConnector\Model\CategoryFactory $eshopsyncCategory,
        \Webkul\SalesforceConnector\Helper\Connection $helperConnection,
        // Logger $logger,
        \Magento\Backend\Model\UrlInterface $backendUrl,
        \Magento\Framework\Json\Helper\Data $jsonHelper,
        \Webkul\SalesforceConnector\Helper\Data $helperData,
        \Magento\Framework\Registry $registry
    ) {
        $this->_eventManager = $eventManager;
        $this->_catalogCategory = $catalogCategory;
        $this->_image = $image;
        $this->_eshopsyncCategory = $eshopsyncCategory;
        $this->_helperConnection = $helperConnection;
        $this->_helperData = $helperData;
        // $this->_logger = $logger;
        $this->_backendUrl = $backendUrl;

        $this->_registry = $registry;

        /* adding for testing */
        $this->_json = $jsonHelper;

        parent::__construct($context);
    }

    public function testMethod()
    {
        $this->_helperConnection->hitApi('Webkul/mgEshop/');
    }

    public function categoryMapping($data, $error = 0)
    {
        $model = $this->_eshopsyncCategory->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $data['magento_id'])->getData();
        if (count($collection) == 1) {
            $data['entity_id'] = $collection[0]['entity_id'];
        } elseif (count($collection) > 1) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
        $model->setData($data);
        $model->save();
    }

    public function deleteMapping($id, $error = 0)
    {
        $model = $this->_eshopsyncCategory->create();
        $collection = $model->getCollection()->addFieldToFilter('magento_id', $id)->getData();
        if (count($collection) > 0) {
            foreach ($collection as $row) {
                $model->setId($row['entity_id'])->delete();
            }
        }
    }

    public function exportCategories($categoryIds, $update = false)
    {
        $str = '';
        $checkInExistingFlag = false;
        $catArr = $imageCatArr = $imageUrlArr = $mapping_data = [];

        try {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $categoryCollection = $objectManager->create(\Magento\Catalog\Model\ResourceModel\Category\Collection::class);
            $collection = $categoryCollection->addFieldToFilter('entity_id', ['in' => $categoryIds]);
            foreach ($collection as $category) {
                $category->load($category['entity_id']);

                $sObject = new \stdclass();
                $name = $category->getName();
                $sObject->Name = $name;
                $sObject->webkul_es_mage__Page_Title__c = $category->getMetaTitle();
                $sObject->webkul_es_mage__Meta_Keywords__c = $category->getMetaKeywords();
                $sObject->webkul_es_mage__Meta_Description__c = $category->getMetaDescription();
                $sObject->webkul_es_mage__Category_description__c = strip_tags($category->getDescription());
                $imageUrl = $category->getImageUrl();
                if ($imageUrl) {
                    $imagePath = pathinfo($imageUrl);
                    $title = $imagePath['basename'];
                    $imageUrlArr[$category['entity_id']] = [
                        'url' => $imageUrl,
                        'title' => $title,
                    ];
                    $imageCatArr[$category['entity_id']] = $sObject;
                } else {
                    $catArr[] = [
                        'method' => 'PATCH',
                        'referenceId' => 'cat_' . $category['entity_id'],
                        'url' => '/services/data/v46.0/sobjects/webkul_es_mage__categories__c/webkul_es_mage__Category_ID__c/' . $category['entity_id'],
                        'body' => $sObject,
                    ];
                }
            }

            if (count($imageUrlArr) > 0) {
                $syncToFile = $this->_helperData->getConfigValue('catPdt', 'imgToFile');
                if ($syncToFile) {
                    $existingFiles = $this->_helperData->checkMultiFileExist(array_column($imageUrlArr, 'title'));

                    if (isset($existingFiles) && count($existingFiles) > 0) {
                        $checkInExistingFlag = true;
                    }
                    foreach ($imageUrlArr as $catId => $value) {
                        $sObject = new \stdClass();
                        $searchkey = array_search($value['title'], array_column($existingFiles, 'Title'));
                        if ($checkInExistingFlag && count($existingFiles) > 0 && $searchkey) {
                            $sObject = $imageCatArr[$catId];
                            $sObject->webkul_es_mage__Category_Image_ID__c = $existingFiles[$searchkey]['Id'];
                            $catArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'cat_' . $catId,
                                'url' => '/services/data/v46.0/sobjects/webkul_es_mage__categories__c/webkul_es_mage__Category_ID__c/' . $catId,
                                'body' => $sObject,
                            ];
                        } else {
                            $mediaImage = [];
                            $mediaImage['Title'] = $value['title'];
                            $mediaImage['PathOnClient'] = $value['url'];
                            $mediaImage['IsMajorVersion'] = false;
                            $mediaImage['VersionData'] = base64_encode(file_get_contents($value['url']));
                            $catArr[] = [
                                'method' => 'POST',
                                'referenceId' => 'image_' . $catId,
                                'url' => '/services/data/v46.0/sobjects/ContentVersion',
                                'body' => $mediaImage,
                            ];
                            $sObject = $imageCatArr[$catId];
                            $sObject->webkul_es_mage__Category_Image_ID__c = '@{image_' . $catId . '.id}';
                            $catArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'cat_' . $catId,
                                'url' => '/services/data/v46.0/sobjects/webkul_es_mage__categories__c/webkul_es_mage__Category_ID__c/' . $catId,
                                'body' => $sObject,
                            ];
                            $catArr[] = [
                                'method' => 'GET',
                                'referenceId' => 'contDoc_' . $catId,
                                'url' => "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'@{image_" . $catId . ".id}'",
                            ];
                            $catArr[] = [
                                'method' => 'POST',
                                'referenceId' => 'imagelink_' . $catId,
                                'url' => '/services/data/v46.0/sobjects/ContentDocumentLink',
                                'body' => [
                                    'LinkedEntityId' => '@{cat_' . $catId . '.id}',
                                    'ShareType' => 'I',
                                    'Visibility' => 'AllUsers',
                                    'ContentDocumentId' => '@{contDoc_' . $catId . '.records[0].ContentDocumentId}',
                                ],
                            ];
                        }
                    }
                } else {
                    $existingDocs = $this->_helperData->checkMultiDocExist(array_column($imageUrlArr, 'title'));
                    if (isset($existingDocs) && count($existingDocs) > 0) {
                        $checkInExistingFlag = true;
                    }
                    foreach ($imageUrlArr as $catId => $value) {
                        $documentMedia = new \stdClass();
                        $name = $value['title'];
                        $documentMedia->Name = $name;
                        $documentMedia->IsPublic = true;
                        $docFolder = $this->_helperData->getConfigValue('catPdt', 'docFolder');
                        $documentMedia->FolderId = $this->_json->jsonDecode($docFolder)['id'];
                        $documentMedia->Body = base64_encode(file_get_contents($value['url']));
                        $sfFileId = array_search($value['title'], $existingDocs);
                        if ($sfFileId) {
                            $catArr[] = [
                                'method' => 'PATCH',
                                'referenceId' => 'image_' . $catId,
                                'url' => '/services/data/v46.0/sobjects/Document/Id/' . $sfFileId,
                                'body' => $documentMedia,
                            ];
                        } else {
                            $catArr[] = [
                                'method' => 'POST',
                                'referenceId' => 'image_' . $catId,
                                'url' => '/services/data/v46.0/sobjects/Document/',
                                'body' => $documentMedia,
                            ];
                        }
                        $sObject = $imageCatArr[$catId];
                        $sObject->webkul_es_mage__Category_Image_ID__c = '@{image_' . $catId . '.id}';
                        $catArr[] = [
                            'method' => 'PATCH',
                            'referenceId' => 'cat_' . $catId,
                            'url' => '/services/data/v46.0/sobjects/webkul_es_mage__categories__c/webkul_es_mage__Category_ID__c/' . $catId,
                            'body' => $sObject,
                        ];
                    }
                }
            }
            $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $catArr]);
            $skipped = $errored = $succeded = 0;
            if (isset($response) && isset($response['compositeResponse'])) {
                foreach ($response['compositeResponse'] as $data) {
                    if (strpos($data['referenceId'], 'cat_') !== false) {
                        $magentoId = ltrim($data['referenceId'], 'cat_');
                        if (isset($data['body']['success'])) {
                            if (!$update) {
                                $mapping_data[] = [
                                    'magento_id' => $magentoId,
                                    'sforce_id' => $data['body']['id'],
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 1,
                                    'error' => '',
                                ];
                            } else {
                                $this->categoryMapping([
                                    'magento_id' => $magentoId,
                                    'sforce_id' => $data['body']['id'],
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 1,
                                    'error' => '',
                                ]);
                            }
                            ++$succeded;
                        } elseif (!isset($data['body']['success']) && isset($data['body'][0]['errorCode'])) {
                            if (!$update) {
                                $mapping_data[] = [
                                    'magento_id' => $magentoId,
                                    'sforce_id' => '',
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 0,
                                    'error' => $data['body'][0]['message'],
                                ];
                            } else {
                                $this->categoryMapping([
                                    'magento_id' => $magentoId,
                                    'sforce_id' => '',
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 0,
                                    'error' => $data['body'][0]['message'],
                                ]);
                            }
                            ++$errored;
                        } else {
                            ++$skipped;
                        }
                    }
                }
            }

            if (count($mapping_data) > 0) {
                if (!$update) {
                    $this->_helperData->bulkInsert('wk_salesforce_eshopsync_category_mapping', $mapping_data);
                }
            }

            return ['skip' => $skipped, 'err' => $errored, 'success' => $succeded];
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
            return ['skip' => 0, 'err' => 0, 'success' => 0];
        }
    }

    public function getCategoryArray($categoryId)
    {
        try {
            $reqArr = $mediaLink = [];
            $imageExist = false;
            $category = $this->_catalogCategory->create()->load($categoryId);
            $sObject = new \stdclass();
            $name = $category->getName();
            $sObject->Name = $name;
            // $sObject->webkul_es_mage__Category_ID__c = $categoryId;
            $sObject->webkul_es_mage__Page_Title__c = $category->getMetaTitle();
            $sObject->webkul_es_mage__Meta_Keywords__c = $category->getMetaKeywords();
            $sObject->webkul_es_mage__Meta_Description__c = $category->getMetaDescription();
            $sObject->webkul_es_mage__Category_description__c = strip_tags($category->getDescription());
            $imageUrl = $category->getImageUrl();
            if ($imageUrl) {
                $imageId = $this->_helperData->uploadImageToSalesForceFile('category', $categoryId, $imageUrl, $reqArr);
                if ($imageId) {
                    if ($imageId === true) {
                        $sObject->webkul_es_mage__Category_Image_ID__c = '@{image_' . $categoryId . '.id}';
                        if ($this->_helperData->getConfigValue('catPdt', 'imgToFile')) {
                            $mediaLink = [
                                'method' => 'POST',
                                'referenceId' => 'imagelink_' . $categoryId,
                                'url' => '/services/data/v46.0/sobjects/ContentDocumentLink',
                                'body' => [
                                    'LinkedEntityId' => '@{cat_' . $categoryId . '.id}',
                                    'ShareType' => 'I',
                                    'Visibility' => 'AllUsers',
                                    'ContentDocumentId' => '@{contDoc.records[0].ContentDocumentId}',
                                ],
                            ];
                        }
                    } else {
                        $sObject->webkul_es_mage__Category_Image_ID__c = $imageId;
                        $imageExist = true;
                    }
                }
            }
            $reqArr[] = [
                'method' => 'PATCH',
                'referenceId' => 'cat_' . $categoryId,
                'url' => '/services/data/v46.0/sobjects/webkul_es_mage__categories__c/webkul_es_mage__Category_ID__c/' . $categoryId,
                'body' => $sObject,
            ];
            if (count($mediaLink) > 0) {
                $reqArr[] = [
                    'method' => 'GET',
                    'referenceId' => 'contDoc',
                    'url' => $imageExist ? "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'" . $imageId . "'" : "/services/data/v46.0/query/?q=SELECT+ContentDocumentId+FROM+ContentVersion+WHERE+Id+=+'@{image_" . $categoryId . ".id}'",
                ];
                $reqArr[] = $mediaLink;
            }
        } catch (\Exception $e) {
            $this->_helperData->eshopsyncLog($e->getMessage());
        }

        return $reqArr;
    }

    public function exportSpecificCategory($magentoId)
    {
        $sforceId = false;
        if ($magentoId) {
            $catArr = $this->getCategoryArray($magentoId);
            try {
                $response = $this->_helperConnection->compositeUpsertSObject(['compositeRequest' => $catArr]);
                //echo"<pre>";var_dump($catArr,$response);die;
                if (isset($response['compositeResponse'])) {
                    foreach ($response['compositeResponse'] as $data) {
                        $mapping_data = [];
                        if (strpos($data['referenceId'], 'cat_') !== false) {
                            if (isset($data['body']['success'])) {
                                $sforceId = $data['body']['id'];
                                $mapping_data = [
                                    'magento_id' => $magentoId,
                                    'sforce_id' => $data['body']['id'],
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 1,
                                    'error' => '',
                                ];
                            } elseif (isset($data['body'][0]['message'])) {
                                $mapping_data = [
                                    'magento_id' => $magentoId,
                                    'sforce_id' => 0,
                                    'created_by' => $this->_helperData->getCurrentUser(),
                                    'sync_status' => 0,
                                    'error' => $data['body'][0]['message'],
                                ];
                            }
                            if (count($mapping_data) > 0) {
                                $this->categoryMapping($mapping_data);
                            }
                        }
                    }
                }
                //$sforceId = $this->_helperConnection->upsertSObject('webkul_es_mage__categories__c','webkul_es_mage__Category_ID__c',$magentoId ,$sObject);

                // if($sforceId && isset($sObject->webkul_es_mage__Category_Image_ID__c) && $syncToFile)
                //     $this->_helperData->linkContentDocToSalesForce($sforceId, $sObject->webkul_es_mage__Category_Image_ID__c);
            } catch (\Exception $e) {
                $error = 'Export Error, Category Id ' . $magentoId . ' >>' . $e->getMessage();
                $error_mapping_data = [
                    'magento_id' => $magentoId,
                    'sync_status' => 0,
                    'error' => 'Export Error, Category Id ' . $magentoId . ' >>' . $e->getMessage(),
                ];
                $this->categoryMapping($error_mapping_data);
                $this->_helperData->eshopsyncLog($error);
            }
        }

        return $sforceId;
    }
}
