<?php
/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_Eshopsync
 * @author    Webkul Software Private Limited
 * @copyright Copyright (c) 2010-2017 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\SalesforceConnector\Observer;
 
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as Observer;
use Magento\Framework\App\RequestInterface;
 
class OnOrderAddressSave implements ObserverInterface
{
 
    /** @var \Magento\Framework\Logger\Monolog */
    protected $_logger;

    /** @var \Webkul\Eshopsync\Helper\Connection */
    protected $_helperConnection;
    protected $_helperData;

    /** @var \Webkul\Eshopsync\Helper\Order */
    protected $_helperOrder;

    /** @var Magento\Framework\App\RequestInterface */
    protected $_request;
    protected $_salesOrder;

    /**
     * @param \Psr\Log\LoggerInterface                   $loggerInterface
     * @param \Webkul\Eshopsync\Helper\Connection        $helperConnection
     * @param \Webkul\Eshopsync\Helper\Order             $helperOrder
     * @param RequestInterface                           $requestInterface
     * @param \Webkul\Eshopsync\Model\CategoryFactory    $eshopsyncCategory
     * @param \Magento\Catalog\Model\CategoryFactory     $catalogCategory
     */
    public function __construct(
        \Psr\Log\LoggerInterface $loggerInterface,
        \Webkul\SalesforceConnector\Helper\Connection $helperConnection,
        \Webkul\SalesforceConnector\Helper\Data $helperData,
        \Webkul\SalesforceConnector\Helper\Order $helperOrder,
        \Magento\Sales\Model\OrderFactory $salesOrder,
        RequestInterface $requestInterface
    ) {
    
        $this->_logger = $loggerInterface;
        $this->_helperConnection = $helperConnection;
        $this->_helperData = $helperData;
        $this->_helperOrder = $helperOrder;
        $this->_request = $requestInterface;
        $this->_salesOrder = $salesOrder;
    }
    /**
     * This is the method that fires when the event runs.
     *
     * @param Observer $observer
     */

    public function execute(Observer $observer)
    {   
        $orderSync = $this->_helperData->getConfigValue('autoSync', 'autoSyncOrderSync');
        if ($orderSync) {
            $event = $observer->getEvent();
            $address = $event->getDataObject();

            /** @var int $orderId */
            $orderId = $address->getParentId();
            $addressType = $address->getAddressType();

            if (empty($orderId) || !$orderId) {
                return;
            } 
            $thisOrder = $this->_salesOrder->create()->load($orderId);
            $billingAddress = $thisOrder->getBillingAddress();
            if($billingAddress != null)
            $this->_helperOrder->syncSpecificOrder($orderId, $thisOrder);  
               
           
            
        }
    }
}
